object Bench {

  var startTime = 0L
  var stopTime = 0L

  def start() = {
    startTime = System.nanoTime()
  }

  def stop() = {
    stopTime = System.nanoTime()
  }

  var times: Map[String,Double] = Map()
  var mintimes: Map[String,Double] = Map()
  var maxtimes: Map[String,Double] = Map()

  def time[R](s: String)(block: => R): R = {
    val t0 = System.nanoTime()
    val result = block    // call-by-name
    val t1 = System.nanoTime()
    mintimes = mintimes.updated(s,Math.min(mintimes.getOrElse(s,Double.MaxValue),t1 - t0))
    maxtimes = maxtimes.updated(s,Math.max(maxtimes.getOrElse(s,0.0),t1 - t0))
    times = times.updated(s,times.getOrElse(s,0.0) + t1 - t0)
    result
  }

  def report() = {
    if (!times.isEmpty) {
      val maxsize = times.map(_._1.size).max
      println("====== REPORT ======")
      println(times.map { case (s:String,t:Double) => "== %s: %.2fs\t%.2fs\t%.2fs".
        format(s.padTo(maxsize,' '),t/1000000000.0,mintimes(s)/1000000000.0,maxtimes(s)/1000000000.0)
      }.toList.sorted.map(s => (1 to s.count(_=='/')).map("  ").mkString + s).mkString("\n"))
      println("Total time: " + (stopTime - startTime)/1000000000.0)
      println("====================")
    }
  }
}
